"""
    = Vina =

    This plugin does virtual screening with the AutoDock Vina software. It also
    uses OpenBabel for conversion from SMILES to PDBQT and AutoDockTools for
    input preparation. This software takes advantage of the new Qt graphical
    interface of PyMOL 2.x.

    @author Pedro Sousa Lacerda
    @email pslacerda@gmail.com
    @license BSD-2-Clause
    @credits Laboratório de Bioinformática e Modelagem Molecular

    == Usage ==

    A new "Vina" menu will appear at the top of the window. Click there.

    === ''Run Docking'' menu ===

    Input text fields ('''Target''', '''Flexible chains''' and '''Box''') are
    to be filled with selections. <code>polymer</code> is a good selection for
    target and <code>polymer within 4 of (name CU and chain A)</code> is an
    example box for PDB 4OUA. But probably you want create selections yourself
    through the GUI (eg. "sele" or "sele_target") and name them on input fields.

    The remaining inputs are AutoDock Vina options.
    [http://vina.scripps.edu/manual.html#usage Read the manual] for more
    information.

    The '''Ligand file''' button is to load a SMILES input file. The
    '''Output folder''' set the folder where the results will be saved.

    === ''Load Docking'' menu ===

    Load and analyze a virtual screening by opening the <code>docking.json</code>
    file at the output folder. Choose a title and a max number of modes
    (remember that a single molecule can have many binding modes), then choose
    the max rank position.

    For instance if you want to see only the first binding mode for the top 50
    molecules, then choose respectively 50 and 1. Or if you want to see the
    top 3 modes up to 40 molecules choose 40 and 3. Note that sometimes a
    molecule can have only one or two binding modes. The '''Polar contacts'''
    option is a option that calls the [[distance]] command to show possible
    polar contacts between the target and the docked molecules, it is better
    visualized with the target represented as sticks.

    == Bugs & feature requests ==

    Report any bugs or feature requests at the
    [https://github.com/Pymol-Scripts/Pymol-script-repo/issues | pymol-script-repo]
"""

import itertools
import json
import os
import shlex
import shutil
import subprocess
import textwrap
from contextlib import contextmanager
from glob import glob
from operator import itemgetter
from os.path import basename, dirname, expanduser, splitext

import numpy as np
import pymol
import pymol.gui
from pymol import cmd
from pymol.cgo import COLOR, CYLINDER, SPHERE

from .commons import run

QWidget = pymol.Qt.QtWidgets.QWidget
QFileDialog = pymol.Qt.QtWidgets.QFileDialog
QFormLayout = pymol.Qt.QtWidgets.QFormLayout
QPushButton = pymol.Qt.QtWidgets.QPushButton
QSpinBox = pymol.Qt.QtWidgets.QSpinBox
QDoubleSpinBox = pymol.Qt.QtWidgets.QDoubleSpinBox
QDockWidget = pymol.Qt.QtWidgets.QDockWidget
QLineEdit = pymol.Qt.QtWidgets.QLineEdit
QCheckBox = pymol.Qt.QtWidgets.QCheckBox
QApplication = pymol.Qt.QtWidgets.QApplication
QMessageBox = pymol.Qt.QtWidgets.QMessageBox
QVBoxLayout = pymol.Qt.QtWidgets.QVBoxLayout
QTextEdit = pymol.Qt.QtWidgets.QTextEdit
QDialog = pymol.Qt.QtWidgets.QDialog
QDialogButtonBox = pymol.Qt.QtWidgets.QDialogButtonBox
QDesktopWidget = pymol.Qt.QtWidgets.QDesktopWidget
QProgressBar = pymol.Qt.QtWidgets.QProgressBar
QAction = pymol.Qt.QtWidgets.QAction
QComboBox = pymol.Qt.QtWidgets.QComboBox

LeftDockWidgetArea = pymol.Qt.QtCore.Qt.LeftDockWidgetArea
QRegExp = pymol.Qt.QtCore.QRegExp
QtCore = pymol.Qt.QtCore
QThread = pymol.Qt.QtCore.QThread
pyqtSignal = pymol.Qt.QtCore.Signal

QRegExpValidator = pymol.Qt.QtGui.QRegExpValidator
QPalette = pymol.Qt.QtGui.QPalette
QTextDocument = pymol.Qt.QtGui.QTextDocument
QIntValidator = pymol.Qt.QtGui.QIntValidator
QTextCursor = pymol.Qt.QtGui.QTextCursor
QIcon = pymol.Qt.QtGui.QIcon


###############################################
#                Utils                        #
###############################################


@contextmanager
def chdir(dir):
    cwd = os.getcwd()
    os.chdir(dir)
    yield
    os.chdir(cwd)


class BaseThread(QThread):

    numSteps = pyqtSignal(int)
    currentStep = pyqtSignal(int)

    logEvent = pyqtSignal(str)
    logCodeEvent = pyqtSignal(str)
    logRawEvent = pyqtSignal(str)

    done = pyqtSignal(bool)

    def __init__(self, *args, parent=None):
        super().__init__(parent)
        self.args = args


def display_box_sel(name, sel, margin):
    coords = cmd.get_coords(sel)
    max = np.max(coords, axis=0) + margin
    min = np.min(coords, axis=0) - margin
    display_box(name, max, min)


def display_box(name, max_coords, min_coords):
    #
    # From the original AutoDock plugin
    #

    box = [
        [max_coords[0], min_coords[0]],
        [max_coords[1], min_coords[1]],
        [max_coords[2], min_coords[2]],
    ]
    cylinder_size = 0.2
    color = [1.0, 1.0, 1.0]

    view = cmd.get_view()
    obj = []

    # box_color
    for i in range(2):
        for k in range(2):
            for j in range(2):
                if i != 1:
                    obj.append(CYLINDER)
                    obj.extend([box[0][i], box[1][j], box[2][k]])
                    obj.extend([box[0][i + 1], box[1][j], box[2][k]])
                    obj.append(cylinder_size)
                    obj.extend(color)
                    obj.extend(color)
                    obj.append(COLOR)
                    obj.extend(color)
                    obj.append(SPHERE)
                    obj.extend([box[0][i], box[1][j], box[2][k], cylinder_size])

                if j != 1:
                    obj.append(CYLINDER)
                    obj.extend([box[0][i], box[1][j], box[2][k]])
                    obj.extend([box[0][i], box[1][j + 1], box[2][k]])
                    obj.append(cylinder_size)
                    obj.extend(color)
                    obj.extend(color)
                    obj.append(COLOR)
                    obj.extend(color)
                    obj.append(SPHERE)
                    obj.extend([box[0][i], box[1][j + 1], box[2][k], cylinder_size])
                if k != 1:
                    obj.append(CYLINDER)
                    obj.extend([box[0][i], box[1][j], box[2][k]])
                    obj.extend([box[0][i], box[1][j], box[2][k + 1]])
                    obj.append(cylinder_size)
                    obj.extend(color)
                    obj.extend(color)
                    obj.append(COLOR)
                    obj.extend(color)
                    obj.append(SPHERE)
                    obj.extend([box[0][i], box[1][j], box[2][k + 1], cylinder_size])
    axes = [[2.0, 0.0, 0.0], [0.0, 2.0, 0.0], [0.0, 0.0, 2.0]]
    cmd.load_cgo(obj, name)
    cmd.set_view(view)


###############################################
#          Load Result Pannel                 #
###############################################


def parse_vina_log(filename):
    with open(filename) as pdbqt_file:
        for line in pdbqt_file:
            if line.startswith("MODEL"):
                _, mode_txt = line.split()
                mode = int(mode_txt)
                state = "IN_MODE"
            elif line.startswith("REMARK VINA RESULT:"):
                parts = line.split()
                affinity = float(parts[3])
            elif line.startswith("REMARK  Name ="):
                parts = line.split()
                name = parts[3]
                yield {
                    "name": name,
                    "affinity": affinity,
                    "mode": mode,
                    "filename": filename,
                }


def load_vina_results(project_file, group, max_load, max_rank, interactions_check):

    # Load project data
    with open(project_file) as _project_file:
        project_data = json.load(_project_file)

    # Load target
    target_name = f"{group}.target"
    if project_data["flexible"]:
        cmd.load(project_data["rigid_pdbqt"], target_name)
    else:
        cmd.load(project_data["target_pdbqt"], target_name)

    cmd.group(group)
    cmd.group(group, target_name)

    # Show box
    box_name = f"{group}.box"
    display_box(
        box_name,
        (
            project_data["center_x"] + project_data["size_x"] / 2,
            project_data["center_y"] + project_data["size_y"] / 2,
            project_data["center_z"] + project_data["size_z"] / 2,
        ),
        (
            project_data["center_x"] - project_data["size_x"] / 2,
            project_data["center_y"] - project_data["size_y"] / 2,
            project_data["center_z"] - project_data["size_z"] / 2,
        ),
    )
    cmd.group(group, box_name)

    # Parse results
    results_dir = project_data["results_dir"]
    results = itertools.chain.from_iterable(
        map(parse_vina_log, glob(f"{results_dir}/poses/*.pdbqt"))
    )
    results = sorted(results, key=itemgetter("affinity"))

    cache = set()
    objects = set()
    count = 0
    for pose in results:
        # Ignore poses which mode is greater than max
        if pose["mode"] > max_rank:
            continue

        # Load molecule into cache
        cache_name = cmd.get_legal_name(pose["filename"].replace(".", "_"))
        if cache_name not in cache:
            cmd.load(pose["filename"], cache_name)
            cache.add(cache_name)

        # Compute object names
        score = int(-10 * pose["affinity"])
        state = pose["mode"]
        base_name = f'{group}.{pose["name"]}_{pose["mode"]}_{score}'
        obj_name = f"{base_name}.mol"
        polar_name = f"{base_name}.polar"

        # Create group
        cmd.group(base_name)

        # Create molecule object
        cmd.create(obj_name, cache_name, state, 1)
        cmd.group(base_name, obj_name)

        if interactions_check:
            cmd.distance(polar_name, target_name, obj_name, 2)
            cmd.group(base_name, polar_name)

        objects.add(obj_name)
        count += 1
        if count >= max_load:
            break
    cmd.delete("delete " + " ".join(cache))


def new_load_results_widget():
    dockWidget = QDockWidget()
    dockWidget.setWindowTitle("AutoDock Vina: Load Results")

    widget = QWidget()

    layout = QFormLayout(widget)
    widget.setLayout(layout)
    dockWidget.setWidget(widget)

    #
    # Group name
    #
    group_title = QLineEdit("GroupTitle", widget)
    group_title.setValidator(QRegExpValidator(QRegExp("^[0-9A-Za-z_-]+$")))

    #
    # Max number of total loaded poses
    #
    max_load_spin = QSpinBox(widget)
    max_load_spin.setRange(1, 99999999)
    max_load_spin.setValue(15)
    max_load_spin.setGroupSeparatorShown(True)

    #
    # Only the best poses of each ligand
    #
    max_rank_spin = QSpinBox(widget)
    max_rank_spin.setRange(1, 20)
    max_rank_spin.setValue(9)
    max_rank_spin.setGroupSeparatorShown(True)

    #
    # Show interactions contacts
    #
    interactions_check = QCheckBox()
    interactions_check.setChecked(False)

    #
    # Choose output folder
    #
    project_file = None
    project_button = QPushButton("Open docking...", widget)

    @project_button.clicked.connect
    def choose_results_dir():
        nonlocal project_file
        project_file = str(
            QFileDialog.getOpenFileName(
                project_button,
                "Docking file",
                expanduser("~"),
                "Docking project file (docking.json)",
            )[0]
        )
        if not project_file:
            return
        project_button.setText(basename(project_file))

    load_button = QPushButton("Load", widget)

    @load_button.clicked.connect
    def run():
        if not project_file:
            return

        load_vina_results(
            project_file,
            group_title.text(),
            max_load_spin.value(),
            max_rank_spin.value(),
            interactions_check.isChecked(),
        )

    #
    # Setup form
    #
    layout.addRow("Title:", group_title)
    layout.addRow("Max load:", max_load_spin)
    layout.addRow("Max rank:", max_rank_spin)
    layout.addRow("Polar contacts:", interactions_check)
    layout.addWidget(project_button)
    layout.addWidget(load_button)
    widget.setLayout(layout)

    return dockWidget


###############################################
#          Run Docking Pannel                 #
###############################################


def get_flex_selection(flex_sel, target_sel):
    return f"{flex_sel} and {target_sel}"


class VinaThreadDialog(QDialog):
    def __init__(self, *vina_args, parent=None):
        super().__init__(parent)
        self.vina = VinaThread(*vina_args)
        self.vina.done.connect(self._done)

        # Setup window
        self.setModal(True)
        self.resize(QDesktopWidget().availableGeometry(self).size() * 0.7)
        self.setWindowFlags(self.windowFlags() | QtCore.Qt.CustomizeWindowHint)
        self.setWindowFlags(self.windowFlags() & ~QtCore.Qt.WindowCloseButtonHint)

        self.layout = QVBoxLayout(self)

        # Setup progress bar
        self.progress = QProgressBar()
        self.layout.addWidget(self.progress)
        self.progress.setValue(0)
        self.vina.numSteps.connect(self.progress.setMaximum)
        self.vina.currentStep.connect(self.progress.setValue)

        # Rich text output
        self.text = QTextEdit(self)
        self.layout.addWidget(self.text)
        self.text.setReadOnly(True)
        self.vina.logEvent.connect(self._appendHtml)
        self.vina.logCodeEvent.connect(self._appendCodeHtml)

        # Ok / Cancel buttons
        self.button_box = QDialogButtonBox(
            QDialogButtonBox.Ok | QDialogButtonBox.Abort, QtCore.Qt.Horizontal, self
        )
        self.layout.addWidget(self.button_box)
        self.button_box.accepted.connect(self._start)
        self.button_box.rejected.connect(self._abort)

    def _appendHtml(self, html):
        self.text.moveCursor(QTextCursor.End)
        self.text.insertHtml(self._prepareHtml(html))

    def _appendCodeHtml(self, html):
        self.text.moveCursor(QTextCursor.End)
        self.text.insertHtml("<pre>" + self._prepareHtml(html) + "</pre>")

    def _start(self):
        ok_button = self.button_box.button(QDialogButtonBox.Ok)
        ok_button.setDisabled(True)
        self.vina.start()

    def _abort(self):
        self.vina.terminate()
        self.done(QDialog.Rejected)

    def _done(self, success):
        ok_button = self.button_box.button(QDialogButtonBox.Ok)
        abort_button = self.button_box.button(QDialogButtonBox.Abort)

        ok_button.setDisabled(False)
        abort_button.setDisabled(True)

        self.button_box.accepted.disconnect(self._start)

        @self.button_box.accepted.connect
        def _done():
            if success:
                self.accept()
            else:
                self.reject()

    @staticmethod
    def _prepareHtml(html):
        return textwrap.dedent(html)


#
# Run docking software
#
# Assumes that all arguments are ok. For instance the target_sel returns the
# atoms that will be at the final target.pdb file. Only input files and subprocess
# commands will be checked.
#


class VinaThread(BaseThread):
    def run(self):
        (
            results_dir,
            ligands_file,
            target_sel,
            flex_sel,
            box_sel,
            box_margin,
            ph,
            exhaustiveness,
            num_modes,
            energy_range,
            cpu,
            seed,
        ) = self.args

        self.logEvent.emit("<h2>Preparation</h2>")

        #
        # Check if the output
        #

        def proj(path):
            return results_dir + '/' + path

        if os.listdir(proj('/')):
            self.logEvent.emit(
                f"""
                <br/>
                <font color="red">
                    <b>The output folder is not empty!</b>
                </font>
            """
            )

        #
        # Create ligand directory
        #

        ligands_dir = proj("ligands")
        try:
            os.mkdir(ligands_dir)
        except FileExistsError:
            shutil.rmtree(ligands_dir)
            os.mkdir(ligands_dir)

        #
        # Convert SMILES file into PDBQT
        #
        obabel = pymol.plugins.pref_get("LABIMM_OBABEL")
        command = (
            f'"{obabel}" -i smi "{ligands_file}"'
            f" -ph {ph} --gen3d -m"
            f' -O "{ligands_dir}/.pdbqt"'
        )
        output, success = run(command, cwd=results_dir)
        if success:
            self.logEvent.emit(
                f"""
                <br/>
                <br/><b>Ligands converted to PDBQT.</b>
                <br/><b>OpenBabel command:</b> {command}
            """
            )
            self.logCodeEvent.emit(output)
        else:
            self.logEvent.emit(
                f"""
                <br/>
                <br/><b>Ligands conversion to PDBQT failed.</b>
                <br/><b>OpenBabel command:</b> {command}
            """
            )
            self.logCodeEvent.emit(output)
            self.done.emit(False)
            return

        #
        # Rename PDBQT files accordingly to SMILES
        # Be aware that not every SMILES file has a name column
        #
        count = 0
        lineno = 0
        has_names = True
        with open(ligands_file) as smi:
            for line in smi:
                lineno += 1

                # skip empty lines
                if line.strip() == "":
                    continue

                count += 1
                if has_names:
                    try:
                        # it really has names
                        smiles, name = line.split()
                    except:
                        if count != 1 and has_names:
                            self.logEvent.emit(
                                f"""
                                <br/>
                                <br/><b>Inconsistent SMILES naming at molecule line #{lineno}.</b>
                                <br/><b><i>Please check you SMILES file.</i></b>
                            """
                            )
                            self.done.emit(False)
                            return

                        # first line don't have name
                        # don't rename files
                        has_names = False
                        continue
                    shutil.move(
                        f"{ligands_dir}/{count}.pdbqt",
                        f"{ligands_dir}/{name}.pdbqt"
                    )

        if len(glob(f"{ligands_dir}/*.pdbqt")) != count:
            # The number of generated ligands and SMILES differ
            self.logEvent.emit(
                f"""
                <br/>
                <br/><b>Number of generated PDBQT files and SMILES molecules differ.</b>
                <br/><b>Please check you SMILES file.</b>
            """
            )
            self.done.emit(False)
            return

        #
        # The number of dockings to do
        #
        n_ligands = count
        self.numSteps.emit(count)

        #
        # Prepare rigid target
        #
        target_pdb = proj("target.pdb")
        cmd.save(target_pdb, target_sel)

        adt_python = pymol.plugins.pref_get("LABIMM_ADT_PYTHON")
        prepare_target = pymol.plugins.pref_get("LABIMM_PREPARE_RECEPTOR")
        command = f'"{adt_python}"' f' "{prepare_target}" -r "{target_pdb}"'
        output, success = run(command, cwd=dirname(target_pdb))
        if success:
            self.logEvent.emit(
                f"""
                <br/>
                <br/><b>Rigid target prepared.</b>
                <br/><b>AutoDock command:</b> {command}
            """
            )
            self.logCodeEvent.emit(output)
        else:
            self.logEvent.emit(
                f"""
                <br/>
                <br/><b>Rigid target preparation failed.</b>
                <br/><b>AutoDock command:</b> {command}
            """
            )
            self.logCodeEvent.emit(output)
            self.done.emit(False)
            return

        #
        # Prepare flexible target
        #
        if flex_sel != "":
            #
            # Construct residues string
            #
            flex_residues = set()
            for atom in cmd.get_model(flex_sel).atom:
                flex_residues.add(f"{atom.chain}:{atom.resn}{atom.resi}")

            flex_residues = ",".join(f"target:{res}" for res in flex_residues)

            #
            # Run AutoDock command
            #

            target_pdbqt = proj("target.pdbqt")
            adt_python = pymol.plugins.pref_get("LABIMM_ADT_PYTHON")
            prepare_flexreceptor = pymol.plugins.pref_get(
                "LABIMM_PREPARE_FLEXRECEPTOR"
            )
            command = (
                f'"{adt_python}"'
                f'"{prepare_flexreceptor}"'
                f' -r "{target_pdbqt}"'
                f" -s {flex_residues}"
            )
            output, success = run(command, cwd=dirname(target_pdb))
            if success:
                self.logEvent.emit(
                    f"""
                    <br/>
                    <br/><b>Flexible target prepared.</b>
                    <br/><b>AutoDock command:</b> {command}
                """
                )
                self.logCodeEvent.emit(output)
            else:
                self.logEvent.emit(
                    f"""
                    <br/>
                    <br/><b>Flexible target preparation failed.</b>
                    <br/><b>AutoDock command:</b> {command}
                """
                )
                self.logCodeEvent.emit(output)
                self.done.emit(False)
                return

        #
        # Create Vina results directory
        #

        output_dir = proj("poses")
        try:
            os.mkdir(output_dir)
        except FileExistsError:
            pass

        #
        # Compute box variables
        #
        box_coords = cmd.get_coords(box_sel)

        max = np.max(box_coords, axis=0) + box_margin
        min = np.min(box_coords, axis=0) - box_margin

        half_size = (max - min) / 2
        center = min + half_size

        size_x, size_y, size_z = half_size * 2
        center_x, center_y, center_z = center

        size_x, size_y, size_z = (
            round(float(size_x), 2),
            round(float(size_y), 2),
            round(float(size_z), 2),
        )

        center_x, center_y, center_z = (
            round(float(center_x), 2),
            round(float(center_y), 2),
            round(float(center_z), 2),
        )

        #
        # Project data
        #

        project_file = proj("docking.json")
        project_data = {}

        project_data.update(
            {
                "program": "vina",
                "results_dir": results_dir,
                "ligands_dir": ligands_dir,
                "output_dir": output_dir,
                "size_x": size_x,
                "size_y": size_y,
                "size_z": size_z,
                "center_x": center_x,
                "center_y": center_y,
                "center_z": center_z,
            }
        )

        if flex_sel == "":
            project_data.update(
                {"flexible": False, "target_pdbqt": f"{results_dir}/target.pdbqt"}
            )
        else:
            project_data.update(
                {
                    "flexible": True,
                    "rigid_pdbqt": f"{results_dir}/target_rigid.pdbqt",
                    "flex_pdbqt": f"{results_dir}/target_flex.pdbqt",
                }
            )
        #
        # Prompt for user confirmation
        #

        vina = pymol.plugins.pref_get("LABIMM_VINA")
        base_command = (
            f'"{vina}"'
            f" --center_x {center_x}"
            f" --center_y {center_y}"
            f" --center_z {center_z}"
            f" --size_x {size_x}"
            f" --size_y {size_y}"
            f" --size_z {size_z}"
            f" --cpu {cpu}"
            f" --seed {seed}"
            f" --exhaustiveness {exhaustiveness}"
            f" --num_modes {num_modes}"
            f" --energy_range {energy_range}"
        )
        if project_data["flexible"]:
            rigid_pdbqt = project_data["rigid_pdbqt"]
            flex_pdbqt = project_data["flex_pdbqt"]
            base_command += f' --receptor "{rigid_pdbqt}"' f' --flex "{flex_pdbqt}"'
        else:
            target_pdbqt = project_data["target_pdbqt"]
            base_command += f' --receptor "{target_pdbqt}"'
        self.logEvent.emit(
            f"""
            <br/>
            <h2>Docking</h2>
            <br/>
            <b>Vina base command:</b> {base_command}
        """
        )

        script_command = base_command.replace(results_dir, "./")
        with open(results_dir + '/run_vina_screening.sh', 'w') as script_file:
            script_file.write(
                f"""#!/bin/bash
                for ligand_pdbqt in $(find ligands -name '*.pdbqt')
                do
                    name=$(basename ${{ligand_pdbqt%.pdbqt}})
                    {script_command} \
                        --ligand $ligand_pdbqt \
                        --out poses/$name.out.pdbqt \
                        --log poses/$name.log
                done
            """)

        fail_count = 0
        for i, ligand_pdbqt in enumerate(glob(f"{ligands_dir}/*.pdbqt")):
            name, _ = splitext(basename(ligand_pdbqt))
            output_pdbqt = f"{output_dir}/{name}.out.pdbqt"
            log_txt = f"{output_dir}/{name}.log"

            command = base_command + (
                f' --ligand "{ligand_pdbqt}"'
                f' --out "{output_pdbqt}"'
                f' --log "{log_txt}"'
            )

            output, success = run(command, cwd=results_dir)
            self.currentStep.emit(i + 1)
            if not success:
                fail_count += 1
                if fail_count <= 10:
                    self.logEvent.emit(
                        f"""
                        <br/>
                        <font color="red">
                            <b>Vina command failed:</b> {command}
                            <br/>
                            <pre>{output}</pre>
                        </font>
                    """
                    )
                elif fail_count == 11:
                    self.logEvent.emit(
                        f"""
                        <br/>
                        <h3>
                            <font color="red">
                                Too many errors. Omitting output.
                            </font>
                        <h3>f
                    """
                    )

        done_ligands = len(glob(f"{output_dir}/*.out.pdbqt"))

        self.logEvent.emit("<br/><h2>Summary</h2>")
        summary = f"""
            <br/><b>Total expected runs:</b> {n_ligands}
            <br/><b>Total failures:</b> {fail_count}
            <br/><b>Total found PDBQT files:</b> {done_ligands}
        """
        if done_ligands < n_ligands or fail_count > 0:
            self.logEvent.emit(f"<font color='red'>{summary}</font>")
        else:
            self.logEvent.emit(f"{summary}")

        with open(results_dir + f"/docking.json", "w") as docking_file:
            json.dump(project_data, docking_file, indent=4)
        self.done.emit(True)


def new_run_docking_widget():
    dockWidget = QDockWidget()
    dockWidget.setWindowTitle("AutoDock Vina: Run")

    widget = QWidget()

    layout = QFormLayout(widget)
    widget.setLayout(layout)
    dockWidget.setWidget(widget)

    #
    # Receptor selection
    #
    target_sel = QLineEdit("", widget)

    @target_sel.textEdited.connect
    def validate(text):
        validate_target_sel()

    target_action = None

    def validate_target_sel():
        nonlocal target_action
        text = target_sel.text()
        valid = True
        try:
            if cmd.count_atoms(f"{text}") == 0:
                raise
        except:
            valid = False

        if valid:
            if target_action is not None:
                target_sel.removeAction(target_action)
        else:
            if target_action is not None:
                target_sel.removeAction(target_action)
            target_action = target_sel.addAction(
                QIcon.fromTheme("dialog-error"), QLineEdit.TrailingPosition
            )
        return valid

    #
    # Flexible residues selection
    #
    flex_sel = QLineEdit("", widget)

    @flex_sel.textEdited.connect
    def validate(text):
        validate_flex_sel()

    flex_action = None

    def validate_flex_sel():
        nonlocal flex_action

        text = flex_sel.text()

        if text.strip() == "":
            valid = True
        else:
            try:
                if cmd.count_atoms(f"({text}) and ({target_sel.text()})") == 0:
                    raise
                valid = True
            except:
                valid = False
        if valid:
            if flex_action is not None:
                flex_sel.removeAction(flex_action)
        else:
            if flex_action is not None:
                flex_sel.removeAction(flex_action)
            flex_action = flex_sel.addAction(
                QIcon.fromTheme("dialog-error"), QLineEdit.TrailingPosition
            )
        return valid

    #
    # Box selection
    #
    box_sel = QLineEdit("", widget)

    @box_sel.textEdited.connect
    def validate(text):
        validate_box()

    box_margin_spin = QDoubleSpinBox(widget)
    box_margin_spin.setRange(0.0, 10.0)
    box_margin_spin.setValue(0.0)
    box_margin_spin.setSingleStep(0.1)
    box_margin_spin.setDecimals(1)

    @box_margin_spin.valueChanged.connect
    def validate(value):
        validate_box()

    box_action = None

    def validate_box():
        nonlocal box_action

        text = box_sel.text()
        try:
            if cmd.count_atoms(text) == 0:
                raise Exception
        except:
            cmd.delete("vina_box")
            if box_action is not None:
                box_sel.removeAction(box_action)
            box_action = box_sel.addAction(
                QIcon.fromTheme("dialog-error"), QLineEdit.TrailingPosition
            )
            return False
        cmd.delete("vina_box")
        display_box_sel("vina_box", text, box_margin_spin.value())

        if box_action is not None:
            box_sel.removeAction(box_action)
            box_action = None

        return True

    #
    # Miscellaneous options
    #
    ph_spin = QDoubleSpinBox(widget)
    ph_spin.setRange(0.0, 14.0)
    ph_spin.setValue(7.0)
    ph_spin.setSingleStep(0.1)
    ph_spin.setDecimals(1)

    exhaustiveness_spin = QSpinBox(widget)
    exhaustiveness_spin.setRange(1, 50)
    exhaustiveness_spin.setValue(8)

    num_modes_spin = QSpinBox(widget)
    num_modes_spin.setRange(1, 20)
    num_modes_spin.setValue(9)

    energy_range_spin = QDoubleSpinBox(widget)
    energy_range_spin.setRange(1.0, 10.0)
    energy_range_spin.setValue(3.0)

    cpu_count = QThread.idealThreadCount()
    cpu_spin = QSpinBox(widget)
    cpu_spin.setRange(1, cpu_count)
    cpu_spin.setValue(cpu_count)

    seed_spin = QSpinBox(widget)
    seed_spin.setRange(0, 10000)
    seed_spin.setValue(0)

    #
    # Choose ligand files and run docking
    #
    ligands_file = None
    ligands_button = QPushButton("Choose file...", widget)

    @ligands_button.clicked.connect
    def choose_ligands():
        nonlocal ligands_file
        ligands_file = str(
            QFileDialog.getOpenFileName(
                ligands_button, "Ligand files", expanduser("~"), "SMILES (*.smi *.txt)"
            )[0]
        )
        if not ligands_file:
            return

        ligands_button.setText(basename(ligands_file))

    #
    # Choose output folder
    #
    results_dir = None
    results_button = QPushButton("Choose folder...", widget)

    @results_button.clicked.connect
    def choose_results_dir():
        nonlocal results_dir
        results_dir = str(
            QFileDialog.getExistingDirectory(
                results_button,
                "Output folder",
                expanduser("~"),
                QFileDialog.ShowDirsOnly,
            )
        )
        if not results_dir:
            return

        results_button.setText(basename(results_dir))

    button = QPushButton("Run", widget)

    @button.clicked.connect
    def run():
        if not (validate_target_sel() & validate_flex_sel() & validate_box()):
            return

        if not (results_dir and ligands_file):
            return

        dialog = VinaThreadDialog(
            results_dir,
            ligands_file,
            target_sel.text(),
            flex_sel.text(),
            box_sel.text(),
            box_margin_spin.value(),
            ph_spin.value(),
            exhaustiveness_spin.value(),
            num_modes_spin.value(),
            energy_range_spin.value(),
            cpu_spin.value(),
            seed_spin.value(),
        )
        dialog.exec_()

    layout.addRow("Target:", target_sel)
    layout.addRow("Flexible chains:", flex_sel)
    layout.addRow("Box:", box_sel)
    layout.addRow("Box margin:", box_margin_spin)
    layout.addRow("Ligand pH:", ph_spin)
    layout.addRow("Exhaustiveness:", exhaustiveness_spin)
    layout.addRow("Number of modes:", num_modes_spin)
    layout.addRow("Energy range:", energy_range_spin)
    layout.addRow("Number of CPUs:", cpu_spin)
    layout.addRow("Seed number:", seed_spin)
    layout.addRow("Ligand file:", ligands_button)
    layout.addRow("Output folder:", results_button)
    layout.addWidget(button)
    widget.setLayout(layout)

    return dockWidget


def init_plugin(menu):

    window = pymol.gui.get_qtwindow()

    run_docking_widget = new_run_docking_widget()
    run_docking_action = menu.addAction("Run VINA screening")
    window.addDockWidget(LeftDockWidgetArea, run_docking_widget)
    run_docking_widget.hide()

    @run_docking_action.triggered.connect
    def toggle():
        run_docking_widget.show()

    load_results_widget = new_load_results_widget()
    load_results_action = menu.addAction("Load VINA screening results")
    window.addDockWidget(LeftDockWidgetArea, load_results_widget)
    load_results_widget.hide()

    @load_results_action.triggered.connect
    def toggle():
        load_results_widget.show()



import traceback
from os.path import expanduser

import pymol.gui
import pymol.Qt
from pymol import cmd as pm

from .core import load_atlas, load_ftmap

QFileDialog = pymol.Qt.QtWidgets.QFileDialog
QFormLayout = pymol.Qt.QtWidgets.QFormLayout
QPushButton = pymol.Qt.QtWidgets.QPushButton
QLineEdit = pymol.Qt.QtWidgets.QLineEdit
QDialog = pymol.Qt.QtWidgets.QDialog
QRegExp = pymol.Qt.QtCore.QRegExp
QtCore = pymol.Qt.QtCore
QRegExpValidator = pymol.Qt.QtGui.QRegExpValidator


class LoadFTMapServerResultDialog(QDialog):
    def __init__(self, parent=None):
        super().__init__(parent)

        # Setup window
        self.setModal(True)
        self.setWindowFlags(self.windowFlags() | QtCore.Qt.CustomizeWindowHint)

        self.layout = QFormLayout(self)

        # Input id
        self.result_id_line = QLineEdit("", self)
        self.result_id_line.setValidator(QRegExpValidator(QRegExp("^[0-9]+$")))
        self.layout.addRow("FTMap result id:", self.result_id_line)

        # Ok / Cancel buttons
        self.fetch_button = QPushButton("Fetch")
        self.layout.addWidget(self.fetch_button)
        self.fetch_button.clicked.connect(self.start)

    def start(self):
        result_id = self.result_id_line.text()
        if not result_id:
            return
        self.fetch_button.setDisabled(True)
        try:
            ret = load_ftmap(result_id)
        except Exception as exc:
            self.done(QDialog.Rejected)
            raise Exception("The result cannot be loaded.")
        self.done(QDialog.Accepted)


def init_plugin_gui(menu):
    load_server_result_action = menu.addAction("Load FTMap server result")

    @load_server_result_action.triggered.connect
    def triggered():
        LoadFTMapServerResultDialog().exec_()

    load_atlas_pdb_action = menu.addAction("Load Atlas PDB")

    @load_atlas_pdb_action.triggered.connect
    def triggered():
        atlas_pdb = QFileDialog.getOpenFileNames(
            menu, "Atlas result file", expanduser("~"), "PDB file (*.pdb)"
        )[0]
        load_atlas(*atlas_pdb)

    load_ftmap_pdb_action = menu.addAction("Load FTMap PDB")

    @load_ftmap_pdb_action.triggered.connect
    def triggered():
        ftmap_pdb = QFileDialog.getOpenFileNames(
            menu, "FTMap result file", expanduser("~"), "PDB file (*.pdb)"
        )[0]
        load_ftmap(*ftmap_pdb)



import hashlib
import random
import shlex
import subprocess
import tempfile
from contextlib import contextmanager

import numpy as np
import pandas as pd
import scipy.spatial
import seaborn as sb
from matplotlib import pyplot as plt
from pymol import cmd as pm
from pymol import stored
from pymol.plugins import pref_get

#
# Generic
#


def pairwise(iter1, iter2):
    """Pairwise element iteration."""
    for item1 in iter1:
        for item2 in iter2:
            yield (item1, item2)


#
# PyMOL specific
#


def new_object_name(prefix="", suffix=""):
    """Return an unused object name."""
    while True:
        seed = str(random.random())
        obj = hashlib.sha224(seed.encode()).hexdigest()
        obj = prefix + obj + suffix
        if obj not in pm.get_object_list():
            return obj


@contextmanager
def disable_feedback(what, level):
    """Disable feedback."""
    pm.feedback("disable", what, level)
    yield
    pm.feedback("enable", what, level)


@contextmanager
def settings(**kwargs):
    """Set options temporarily."""
    orig = {}
    for key, value in kwargs.items():
        orig[key] = pm.get(key)
        pm.set(key, value)
    yield
    for key, value in orig.items():
        pm.set(key, value)


#
# Atomic helpers
#


def contact_matrix(a, b, radius=None):
    """
    Compute the contact matrix between A and B.
    When radius is None the contact is defined by:
        Distance(a, b) < VDW(a) + VDW(b)
    When radius is a number the contact is defined by:
        Distance(a, b) < radius
    """
    d = scipy.spatial.distance_matrix(a[["x", "y", "z"]], b[["x", "y", "z"]])
    if radius is None:
        return ((d - b["vdw"].values).T - a["vdw"].values).T <= 0
    else:
        return (d - radius) <= 0


def get_atoms(sel, attrs, state=1):
    """Get the atoms and attributes of a selection."""
    coords = None
    if "coords" in attrs:
        coords = pm.get_coords(sel, state)
        attrs.remove("coords")
    atoms = pd.DataFrame(coords, columns=["x", "y", "z"])

    if attrs:
        fields_str = ", ".join(attrs)
        stored.atoms = []
        pm.iterate_state(state, sel, f"stored.atoms.append(({fields_str}))")
        atoms = pd.concat([atoms, pd.DataFrame(stored.atoms, columns=attrs)], axis=1)
        del stored.atoms
    return atoms


def count_molecules(selection="all"):
    """
    By Thomas Holder.
    """
    tmpsele = pm.get_unused_name("_tmp")
    count = 0
    if pm.select(tmpsele, selection):
        count += 1
        while pm.select(tmpsele, f"{tmpsele} &! bm. first {tmpsele}"):
            count += 1
    pm.delete(tmpsele)
    return count


#
# Etcetera
#


def run(command, **kwargs):
    if isinstance(command, str):
        command = shlex.split(command)
    ret = subprocess.run(command, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, **kwargs)
    output = ret.stdout.decode()
    success = ret.returncode == 0
    return output, success


def rscript(script):
    tmp_fd, tmp_path = tempfile.mkstemp(".R")
    with open(tmp_fd, "w") as tmp_file:
        tmp_file.write(script)
    return run([pref_get("LABIMM_RSCRIPT"), tmp_path])


#
# Commands
#


@pm.extend
def get_fractional_overlap(sel1, sel2, radius=2, state1=1, state2=1, verbose=1):
    """
    Compute the fractional overlap of sel1 respective to sel2.
        FO = Nc/Nt

    Nc is the number of atoms of sel1 in contact with sel2. Nt is the number of atoms
    of sel1.

    Hydrogen atoms are ignored.

    If the contact radius is 0 then the VdW radii will be used.

    The states are for select a single state from a multi-state objects.

    OPTIONS:
        sel1    ligand object.
        sel2    hotspot object.
        radius  the radius so two atoms are in contact (default: 2).
        state1  state of sel1.
        state2  state of sel2.

    EXAMPLES:
        get_fractional_overlap ref_lig, ftmap1234.D.003
        get_fractional_overlap ref_lig, ftmap1234.CS.000_016
    """
    atoms1 = get_atoms(sel1, ["coords", "elem", "vdw"], state1)
    atoms2 = get_atoms(sel2, ["coords", "elem", "vdw"], state2)

    a = atoms1[atoms1["elem"] != "H"]
    b = atoms2[atoms2["elem"] != "H"]
    contacts = np.any(contact_matrix(a, b, radius), axis=1)
    num_contacts = np.sum(contacts)
    total_atoms = len(a)
    fo = num_contacts / total_atoms
    if bool(verbose):
        print(f"  Fractional Overlap = {fo}")
    return fo


@pm.extend
def get_fractional_overlap2(sel1, sel2, radius=2, state1=1, state2=1):
    """
    Compute the fractional overlap.

    SEE:
        get_fractional_overlap
    """
    print(sel1 + " / " + sel2)
    get_fractional_overlap(sel1, sel2, radius, state1, state2, 1)
    print(sel2 + " / " + sel1)
    get_fractional_overlap(sel2, sel1, radius, state2, state1, 1)


@pm.extend
def nearby_aminoacids_similarity(
    sel1,
    sel2,
    polymer1="polymer",
    polymer2="polymer",
    radius=4,
    method="overlap",
    verbose=1,
):
    """
    Compute the overlap coefficient between the aminoacids ids nearby two selections.

    OPTIONS
        sel1        Selection of object 1.
        sel2        Selection of object 2.
        polymer1    protein of sel1.
        polymer2    protein of sel2.
        radius      Radius to look for nearby aminoacids.
        method      'overlap' or 'sorensen–dice'

    EXAMPLES
        nearby_aminoacids_similarity *CS.000_*, *CS.002_*, radius=4
        nearby_aminoacids_similarity *D.001*, *D.002*, polymer1='obj1', polymer2='obj2'
        nearby_aminoacids_similarity 6y84.Bs.001, 6y84.B.004, method=sorensen-dice
    """
    atoms1 = get_atoms(
        f"(polymer and ({polymer1})) within {radius} of ({sel1})", ["chain", "resi"]
    )
    atoms2 = get_atoms(
        f"(polymer and ({polymer2})) within {radius} of ({sel2})", ["chain", "resi"]
    )

    resis1 = set(zip(atoms1.resi, atoms1.chain))
    resis2 = set(zip(atoms2.resi, atoms2.chain))

    if method == "overlap":
        coef = len(resis1.intersection(resis2)) / min(len(resis1), len(resis2))
    elif method == "sorensen-dice":
        coef = 2 * len(resis1.intersection(resis2)) / (len(resis1) + len(resis2))
    else:
        raise Exception("Not supported method.")
    if verbose:
        print("Sel1:", ", ".join(["%s%s" % r for r in resis1]))
        print("Sel2:", ", ".join(["%s%s" % r for r in resis2]))
        print("Similarity coefficient =", coef)
    return coef


@pm.extend
def rms_per_residue(sel1, sel2):
    atoms1 = get_atoms(sel1, ["chain", "resi"])
    atoms2 = get_atoms(sel2, ["chain", "resi"])

    resis1 = set(zip(atoms1.resi, atoms1.chain))
    resis2 = set(zip(atoms2.resi, atoms2.chain))

    labels = []
    values = []
    for resi, chain in resis1:
        labels.append(f"{resi}{chain}")
        values.append(
            pm.rms(
                f"({sel1}) and resi {resi} and chain {chain}",
                f"({sel2}) and resi {resi} and chain {chain}",
            )
        )
    ax = sb.lineplot(labels, values)
    ax.set_title("RMS per residue")
    ax.set_xticklabels(labels, rotation=45, ha="right")
    plt.show()



import subprocess
import sys

import pymol.gui
from pymol.plugins import pref_get

from .commons import rscript
from .fetch_similar.blast_gui import init_gui as fetch_similar_blast_init_gui
from .fetch_similar.shape3d_gui import \
    init_gui as fetch_similar_shape3d_init_gui
from .ftmap.gui import init_plugin_gui as ftmap_init_plugin_gui
from .prefs import guess_prefs
from .vina import init_plugin as vina_init_plugin


def pip(args):
    process = subprocess.Popen(
        [sys.executable, "-m", "pip", "--disable-pip-version-check"] + args,
        universal_newlines=True,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )
    out, err = process.communicate()
    return process, out, err


def init_plugin():
    guess_prefs()

    # FTMap
    window = pymol.gui.get_qtwindow()
    menu_bar = window.menuBar()
    labimm_menu = menu_bar.addMenu("LaBiMM")

    ftmap_init_plugin_gui(labimm_menu)

    # Fetch similar
    fetch_similar_blast_init_gui(labimm_menu)
    fetch_similar_shape3d_init_gui(labimm_menu)
    # Vina
    vina_init_plugin(labimm_menu)

    update_action = labimm_menu.addAction("Update...")

    @update_action.triggered.connect
    def triggered():
        proc, out, err = pip(["install", "--upgrade", "pymol-labimm"])
        if out:
            print(out)
        if err:
            print(err)

        if pref_get("LABIMM_RSCRIPT"):
            out, success = rscript("library(bio3d)")
            if "there is no packaged called" in out:
                out, success = rscript("install.packages('bio3d')")
                print(out)



import fnmatch
import shutil
import tempfile
import textwrap
import webbrowser
from dataclasses import dataclass, field
from glob import glob
from itertools import combinations
from os.path import basename, splitext

import matplotlib.pyplot as plt
import numpy as np
import requests
import scipy.spatial
import seaborn as sb
from cached_property import cached_property
from jinja2 import Template
from pymol import CmdException
from pymol import cmd as pm
from pymol import stored

from ..commons import (count_molecules, disable_feedback, get_atoms,
                       get_fractional_overlap, nearby_aminoacids_similarity,
                       pairwise, settings)

sb.set(font_scale=0.5)


@dataclass
class Cluster:
    """A consensus site."""

    id: str
    sele: str
    coords: np.ndarray = field(repr=False)

    @cached_property
    def selection(self):
        return self.sele

    @cached_property
    def strength(self):
        return count_molecules(self.sele)

    @cached_property
    def max_dist(self):
        return scipy.spatial.distance_matrix(self.coords, self.coords).max()

    # @cached_property
    # def polar_count(self):
    #     stored.resns = []
    #     pm.iterate_state(0, self.selection, "stored.resns.append(resn)")
    #     count = 0
    #     for resn in stored.resns:
    #         if resn in [
    #             "ACD",
    #             "ACN",
    #             "ACT",
    #             "ADY",
    #             "AMN",
    #             "BDY",
    #             "BUT",
    #             "DFO",
    #             "EOL",
    #             "PHN",
    #             "THS",
    #             "URE",
    #         ]:
    #             count += 1
    #     del stored.resns
    #     return count
    #
    # @cached_property
    # def apolar_count(self):
    #     get_atoms()
    #     stored.resns = []
    #     pm.iterate_state(0, self.selection, "stored.resns.append(resn)")
    #     count = 0
    #     for resn in stored.resns:
    #         if resn in ["BEN", "CHX", "DME", "ETH"]:
    #             count += 1
    #     del stored.resns
    #     return count

    def __hash__(self):
        return hash(self.selection)

    @classmethod
    def collect(cls):
        for id in range(0, 50):
            obj_mask = "consensus.{id:03}.*".format(id=id)
            objs = fnmatch.filter(pm.get_object_list(), obj_mask)
            if len(objs) == 0:
                break
            if len(objs) > 1:
                raise Exception(f"Too much objects found: {', '.join(objs)}")
            obj = objs[0]
            pm.flag("ignore", obj, "clear", True)
            coords = pm.get_coords(obj)
            yield Cluster(id, obj, coords)

    @classmethod
    def collect_atlas(cls):
        """Collect cluster objects from an Atlas session."""
        yield from cls.collect()

    @classmethod
    def collect_ftmap(cls):
        """Collect cluster objects from an FTMap session."""
        for obj in pm.get_object_list():
            if obj.endswith(".pdb"):
                new_name = obj[:-4].replace("crosscluster", "consensus")
                pm.set_name(obj, new_name)
        yield from cls.collect()


@dataclass
class Ensemble:
    """A combination of clusters."""

    clusters: [Cluster]

    @cached_property
    def selection(self):
        return " or ".join(f"({c.selection})" for c in self.clusters)

    @cached_property
    def strength(self):
        return sum(cs.strength for cs in self.clusters)

    @cached_property
    def strength0(self):
        return self.clusters[0].strength

    @cached_property
    def max_dist(self):
        return max(
            scipy.spatial.distance_matrix(cs1.coords, cs2.coords).max()
            for (cs1, cs2) in pairwise(self.clusters, self.clusters)
        )

    @cached_property
    def center_to_center(self):
        cd = {}
        for (cs1, cs2) in pairwise(self.clusters, self.clusters):
            if cs1 == cs2:
                cd[(cs1, cs2)] = 0
            else:
                cd[(cs1, cs2)] = scipy.spatial.distance.euclidean(
                    np.average(cs1.coords, axis=0), np.average(cs2.coords, axis=0)
                )
        return cd

    @cached_property
    def max_center_to_center(self):
        return max(self.center_to_center.values())

    @cached_property
    def continuity(self):
        dist = 0
        for (cs1, cs2) in pairwise(self.clusters, self.clusters):
            if cs1.id == cs2.id:
                continue
            dist = max(
                dist,
                scipy.spatial.distance.euclidean(
                    np.average(cs1.coords, axis=0), np.average(cs2.coords, axis=0)
                ),
            )
        return dist

    @cached_property
    def polar_count(self):
        return sum(c.polar_count for c in self.clusters)

    @cached_property
    def apolar_count(self):
        return sum(c.apolar_count for c in self.clusters)

    def __hash__(self):
        return hash(self.selection)

    @classmethod
    def collect(cls, max_size, cluster_collector):
        clusters = list(cluster_collector())
        for sz in range(1, max_size + 1):
            for cs_comb in combinations(clusters, sz):
                yield cls(cs_comb)

    @classmethod
    def collect_atlas(cls, max_size):
        """Collect possible ensembles from an Atlas session."""
        yield from cls.collect(max_size, Cluster.collect_atlas)

    @classmethod
    def collect_ftmap(cls, max_size):
        """Collect possible ensembles from an FTMap session."""
        yield from cls.collect(max_size, Cluster.collect_ftmap)


class Kozakov2015Ensemble(Ensemble):
    """A combination of clusters as described by Kozakov (2015).
    DOI: 10.1021/acs.jmedchem.5b00586
    """

    @cached_property
    def klass(self):
        return (
            "D"
            if self.is_druggable
            else "Dl"
            if self.is_druggable_large
            else "Ds"
            if self.is_druggable_small
            else "B"
            if self.is_borderline
            else "Bl"
            if self.is_borderline_large
            else "Bs"
            if self.is_borderline_small
            else None
        )

    @cached_property
    def is_druggable(self):
        return (
            self.clusters[0].strength >= 16
            and all(cd < 8 for cd in self.center_to_center.values())
            and self.max_dist >= 10
        )

    @cached_property
    def is_druggable_large(self):
        return (
            self.clusters[0].strength >= 16
            and all(cd >= 8 for cd in self.center_to_center.values())
            and self.max_dist >= 10
        )

    @cached_property
    def is_druggable_small(self):
        return (
            self.clusters[0].strength >= 16
            and all(cd < 8 for cd in self.center_to_center.values())
            and 7 <= self.max_dist < 10
        )

    @cached_property
    def is_borderline(self):
        return (
            13 <= self.clusters[0].strength < 16
            and all(cd < 8 for cd in self.center_to_center.values())
            and self.max_dist >= 10
        )

    @cached_property
    def is_borderline_large(self):
        return (
            13 <= self.clusters[0].strength < 16
            and all(cd >= 8 for cd in self.center_to_center.values())
            and self.max_dist >= 10
        )

    @cached_property
    def is_borderline_small(self):
        return (
            13 <= self.clusters[0].strength < 16
            and all(cd < 8 for cd in self.center_to_center.values())
            and 7 <= self.max_dist < 10
        )


@pm.extend
def load_ftmap(
    *paths,
    group=None,
    max_cs=3,
    plot=True,
    plot_annot=True,
    plot_class=None,
    plot_method="overlap",
    table=True,
    _self=pm,
):
    """
    Load a FTMap PDB file and classify hotspot ensembles in accordance to
    Kozakov et al. (2015).
    https://doi.org/10.1021/acs.jmedchem.5b00586

    OPTIONS
        path    PDB file path, glob or server result id.
        group   optional group name to put objects in.
        max_cs  the maximum number of consensus sites to consider.
        plot    plot similarity matrix.
        plot_annot  remove numbers on similarity matrix.
        plot_class  filter to show only a specific hotspot class.
        plot_method see `help nearby_aminoacids_similarity`.

    EXAMPLES
        load_ftmap fftmap.1234.pdb
        load_ftmap fftmap.1111.pdb, fftmap.2222.pdb, group=GRP, max_cs=4
        load_ftmap fftmap_*.pdb, plot_annot=0, plot_class=D
    """
    if all(ch in "1234567890" for ch in paths[0]):
        jobid = paths[0]
        fp, temp = tempfile.mkstemp(suffix=".pdb")
        open(fp).close()
        session = requests.Session()
        session.get("https://ftmap.bu.edu/nousername.php")
        with session.get(
            f"https://ftmap.bu.edu/file.php"
            f"?jobid={jobid}"
            f"&coeffi=0&model=0&filetype=model_file",
            stream=True,
        ) as ret:
            with open(temp, "wb") as fp:
                shutil.copyfileobj(ret.raw, fp)
            if not ret.content:
                raise CmdException(f"Invalid response id {jobid}")
        return process_session(
            Kozakov2015Ensemble.collect_ftmap,
            [temp],
            group,
            int(max_cs),
            plot=plot,
            plot_annot=bool(int(plot_annot)),
            plot_class=plot_class,
            table=table,
            plot_method=plot_method,
            base_root="fftmap" + jobid,
        )
    return process_session(
        Kozakov2015Ensemble.collect_ftmap,
        paths,
        group,
        int(max_cs),
        plot=plot,
        plot_annot=bool(int(plot_annot)),
        plot_class=plot_class,
        table=table,
        plot_method=plot_method,
    )


#
# Plugin commands
#


@pm.extend
def load_atlas(
    *paths,
    group=None,
    max_size=3,
    plot=True,
    plot_annot=True,
    plot_class=None,
    plot_method="overlap",
    table=True,
    _self=pm,
):
    """
    Load an Atlas PDB file. See `help load_ftmap`.
    """
    return process_session(
        Kozakov2015Ensemble.collect_atlas,
        paths,
        group,
        int(max_size),
        plot=plot,
        plot_annot=bool(int(plot_annot)),
        plot_class=plot_class,
        table=table,
        plot_method=plot_method,
    )


def process_session(
    ensemble_collector,
    patterns,
    group,
    max_size,
    plot,
    plot_annot,
    plot_class,
    plot_method,
    table,
    base_root=None,
):
    """Main plugin code."""

    results = {}
    for pattern in sorted(patterns):
        for path in glob(pattern):
            if base_root is None:
                root = pm.get_legal_name(splitext(basename(path))[0])
            else:
                root = base_root
            results[root] = [], []
            ensembles, clusters = results[root]

            if group:
                root = f"{group}.{root}"
            else:
                root = root

            with disable_feedback("all", "warnings"):
                with settings(group_auto_mode=1):
                    pm.load(path)

            try:
                collected_ensembles = list(ensemble_collector(max_size))
                if len(collected_ensembles) == 0:
                    raise Exception("No ensembles found.")
            except:
                raise CmdException(f"File {path} is invalid.")

            with settings(group_auto_mode=2):
                pm.create(f"{root}.protein", "protein")
                pm.delete("protein")

                i = 0

                for ensemble in collected_ensembles:
                    klass = ensemble.klass
                    if klass:
                        i += 1
                        pm.hide("sticks", ensemble.selection)
                        pm.show("line", ensemble.selection)
                        pm.util.cbas(ensemble.selection)

                        obj = f"{root}.{klass}.{i:03}"
                        pm.create(obj, ensemble.selection)

                        if hasattr(pm, "set_property"):
                            pm.set_property("Class", ensemble.klass, obj)
                            pm.set_property("S", ensemble.strength, obj)
                            pm.set_property(
                                "S (CS0)", ensemble.clusters[0].strength, obj
                            )
                            pm.set_property("CD", ensemble.max_center_to_center, obj)
                            pm.set_property("MD", ensemble.max_dist, obj)

                        ensemble.selection = obj
                        ensembles.append(ensemble)

                for i, cluster in enumerate(Cluster.collect_atlas()):
                    pm.hide("sticks", cluster.selection)
                    pm.show("line", cluster.selection)
                    pm.util.cbay(cluster.selection)
                    obj = f"{root}.CS.{i:03}_{cluster.strength:03}"
                    pm.create(obj, cluster.selection)
                    pm.delete(cluster.selection)
                    cluster.selection = obj
                    clusters.append(cluster)

            pm.color("yellow", f"{root}.CS.*")
            pm.color("salmon", f"{root}.B.* or {root}.Bs.* {root}.Bl.*")
            pm.color("red", f"{root}.D.* or {root}.Ds.* {root}.Dl.*")

            pm.hide("lines", f"{root}.*")
            pm.disable(f"{root}.CS")

            pm.show("mesh", f"{root}.B.* or {root}.Bs.* {root}.Bl.*")
            pm.show("mesh", f"{root}.D.* or {root}.Ds.* {root}.Dl.*")

            pm.show("mesh", f"{root}.CS.*")
            pm.hide("nb_spheres", "*label")

            pm.orient(root)

    if plot:

        roots = []
        selections = []

        for root in sorted(results):
            for ensemble in results[root][0]:
                roots.append(root)
                selections.append(ensemble.selection)

        if plot_class:
            selections = [s for s in selections if "." + plot_class + "." in s]

        matrix_sim = np.zeros((len(selections), len(selections)))
        matrix_over = np.zeros((len(selections), len(selections)))
        for i, (root1, selection1) in enumerate(zip(roots, selections)):
            for j, (root2, selection2) in enumerate(zip(roots, selections)):
                matrix_sim[i][j] = nearby_aminoacids_similarity(
                    selection1,
                    selection2,
                    polymer1=root1 + ".protein",
                    polymer2=root2 + ".protein",
                    method=plot_method,
                    verbose=False,
                )
                matrix_over[i][j] = get_fractional_overlap(
                    selection1, selection2, verbose=0
                )

        fig, ax = plt.subplots(1, 2)

        sb.heatmap(
            matrix_sim,
            vmax=1,
            vmin=0,
            xticklabels=selections,
            yticklabels=selections,
            annot=plot_annot,
            cmap="YlGnBu",
            ax=ax[0],
        )
        sb.heatmap(
            matrix_over,
            vmax=1,
            vmin=0,
            yticklabels=selections,
            xticklabels=selections,
            annot=plot_annot,
            cmap="YlGnBu",
            ax=ax[1],
        )
        ax[0].set_title(plot_method + " coefficient")
        ax[0].set_xticklabels(selections, rotation=45, ha="right")
        ax[1].set_title("fractional overlap")
        ax[1].set_xticklabels(selections, rotation=45, ha="right")
        plt.show()

    if table:
        tmpl = Template(
            """
            <table>
                <tr>
                    <th>Object</th>
                    <th>Class</th>
                    <th>S</th>
                    <th>S0</th>
                    <th>CD</th>
                    <th>MD</th>
                </tr>
                {% for root in results %}
                    {% for e in results[root][0] %}
                        <tr>
                            <td>{{ e.selection }}</td>
                            <td>{{ e.klass }}</td>
                            <td>{{ e.strength }}</td>
                            <td>{{ e.strength0 }}</td>
                            <td>{{ e.max_center_to_center | round(2) }}</td>
                            <td>{{ e.max_dist | round(2) }}</td>
                        </tr>
                    {% endfor %}
                {% endfor %}
            </table>
        """
        )

        _, path = tempfile.mkstemp(suffix=".html")
        with open(path, "w") as fd:
            fd.write(tmpl.render(results=results))
        webbrowser.open(path)

    return results


@pm.extend
def calculate_kozakov2015(*args, **kwargs):
    """
    Calculate a hotspot following Kozakov et al (2015).

    USAGE
        calculate_kozakov2015 sel1, ...

    EXAMPLES
        calculate_kozakov2015 *CS.000_*, *CS.002_*
        calculate_kozakov2015 *.000_*, *.001_*

    """
    clusters = []
    for sel in args:
        cluster = Cluster("", sel, pm.get_coords(sel))
        clusters.append(cluster)

    ensemble = Kozakov2015Ensemble(clusters)
    print(
        textwrap.dedent(
            f"""
                {ensemble}
                Class {ensemble.klass}
                S {ensemble.strength}
                S0 {ensemble.strength0}
                CD {ensemble.max_center_to_center}
                MD {ensemble.max_dist}
        """
        )
    )



import pymol.gui
import pymol.Qt

from .shape3d import fetch_similar_shape3d

QWidget = pymol.Qt.QtWidgets.QWidget
QFileDialog = pymol.Qt.QtWidgets.QFileDialog
QFormLayout = pymol.Qt.QtWidgets.QFormLayout
QPushButton = pymol.Qt.QtWidgets.QPushButton
QSpinBox = pymol.Qt.QtWidgets.QSpinBox
QDoubleSpinBox = pymol.Qt.QtWidgets.QDoubleSpinBox
QDockWidget = pymol.Qt.QtWidgets.QDockWidget
QLineEdit = pymol.Qt.QtWidgets.QLineEdit
QCheckBox = pymol.Qt.QtWidgets.QCheckBox
QApplication = pymol.Qt.QtWidgets.QApplication
QMessageBox = pymol.Qt.QtWidgets.QMessageBox
QVBoxLayout = pymol.Qt.QtWidgets.QVBoxLayout
QTextEdit = pymol.Qt.QtWidgets.QTextEdit
QDialog = pymol.Qt.QtWidgets.QDialog
QDialogButtonBox = pymol.Qt.QtWidgets.QDialogButtonBox
QDesktopWidget = pymol.Qt.QtWidgets.QDesktopWidget
QProgressBar = pymol.Qt.QtWidgets.QProgressBar
QAction = pymol.Qt.QtWidgets.QAction
QComboBox = pymol.Qt.QtWidgets.QComboBox

LeftDockWidgetArea = pymol.Qt.QtCore.Qt.LeftDockWidgetArea
QRegExp = pymol.Qt.QtCore.QRegExp
QtCore = pymol.Qt.QtCore
QThread = pymol.Qt.QtCore.QThread
pyqtSignal = pymol.Qt.QtCore.Signal

QRegExpValidator = pymol.Qt.QtGui.QRegExpValidator
QPalette = pymol.Qt.QtGui.QPalette
QTextDocument = pymol.Qt.QtGui.QTextDocument
QIntValidator = pymol.Qt.QtGui.QIntValidator
QTextCursor = pymol.Qt.QtGui.QTextCursor
QIcon = pymol.Qt.QtGui.QIcon


class FetchBlastClusterDialog(QDialog):
    def __init__(self, *vina_args, parent=None):
        super().__init__(parent)

        # Setup window
        self.setWindowFlags(self.windowFlags() | QtCore.Qt.CustomizeWindowHint)

        self.layout = QFormLayout(self)

        # Form
        self.ref_pdb_line = QLineEdit("", self)
        self.ref_pdb_line.setValidator(
            QRegExpValidator(QRegExp("[a-zA-Z0-9]{4}"), self)
        )
        self.layout.addRow("Reference PDB:", self.ref_pdb_line)

        self.similarity_spin = QDoubleSpinBox(self)
        self.similarity_spin.setRange(0.0, 1.0)
        self.similarity_spin.setValue(0.5)
        self.layout.addRow("Min similarity:", self.similarity_spin)

        self.ligand_line = QLineEdit("", self)
        self.layout.addRow("Reference ligand:", self.ligand_line)

        self.dist_spin = QDoubleSpinBox(self)
        self.dist_spin.setRange(0.0, 10.0)
        self.dist_spin.setValue(5.0)
        self.layout.addRow("Distance from ligand:", self.dist_spin)

        self.compounds_line = QLineEdit("organic or inorganic", self)
        self.layout.addRow("Ligand compounds:", self.compounds_line)

        self.prosthetics_line = QLineEdit("HEM FAD NAP NDP ADP FMN", self)
        self.layout.addRow("Prosthetic groups:", self.prosthetics_line)

        self.max_resol_spin = QDoubleSpinBox(self)
        self.max_resol_spin.setRange(0.0, 10.0)
        self.max_resol_spin.setValue(2.0)
        self.layout.addRow("Max resolution:", self.max_resol_spin)

        # Fetch button
        self.fetch_button = QPushButton("Fetch")
        self.layout.addWidget(self.fetch_button)
        self.fetch_button.clicked.connect(self.start)

    def start(self):
        pdb_id = self.ref_pdb_line.text()

        min_similarity = self.similarity_spin.value()
        max_resolution = self.max_resol_spin.value()
        ligand = self.ligand_line.text().strip() or None
        dist = self.dist_spin.value()
        compounds = self.compounds_line.text()
        prosthetic_groups = self.prosthetics_line.text()

        fetch_similar_shape3d(
            pdb_id,
            min_similarity,
            max_resolution,
            ligand,
            dist,
            compounds,
            prosthetic_groups,
        )

        self.done(QDialog.Accepted)


def init_gui(menu):

    action = menu.addAction("Fetch similar (Shape 3D)")

    @action.triggered.connect
    def triggered():
        dialog = FetchBlastClusterDialog()
        dialog.exec_()



from dataclasses import dataclass
from typing import Optional

import requests
from pymol import cmd as pm

from .blast import get_resolution


@dataclass
class SimilarStructure:
    pdb_id: str
    score: float
    resolution: Optional[float]


@pm.extend
def fetch_similar_shape3d(
    pdb_id: str,
    min_similarity: float = 0,
    max_resolution: Optional[float] = None,
    ligand: Optional[str] = None,
    dist: float = 5.0,
    compounds: str = "organic or inorganic",
    prosthetic_groups: str = "HEM FAD NAP NDP ADP FMN",
) -> [SimilarStructure]:
    """
    Fetch similar structures using the 3D-Shape algorithm.

    https://search.rcsb.org/

    OPTIONS:
        pdb_id          Reference PDB id.
        min_similarity  3D-Shape score threshold.
        max_resolution  Fetch only structures up to such resolution.
        ligand          Refrence ligand PDB id for apo evaluation.
        dist            Distance cut-off around reference ligand for apo
                        evaluation. Only used when ligand is given.
        compounds       Selection that shold be considered ligands upon apo
                        evaluation. Only used when ligand is given.
        prosthetic_groups   List of ligands to be ignored when evaluating apo.
    EXAMPLES:
        fetch_similar_shape3d 2XY9
    SEE ALSO:
        fetch_similar_blast
    """
    ret = requests.post(
        url="https://search.rcsb.org/rcsbsearch/v1/query",
        headers={"Content-Type": "application/json"},
        json={
            "query": {
                "type": "terminal",
                "service": "structure",
                "parameters": {
                    "value": {"entry_id": pdb_id, "assembly_id": "1"},
                    "operator": "strict_shape_match",
                },
                "node_id": 1,
            },
            "return_type": "entry",
        },
    )

    similars = []
    for i, result in enumerate(ret.json()["result_set"]):
        sim = SimilarStructure(
            pdb_id=result["identifier"],
            score=result["score"],
            resolution=None,
        )

        # Chek the similarity threshold
        if sim.score < min_similarity:
            continue

        # Check the resolution
        if i >= 1 and max_resolution:
            resol = get_resolution(sim.pdb_id)
            if resol and resol > max_resolution:
                continue
            sim.resolution = resol

        # Fetch the structure
        pm.fetch(sim.pdb_id)

        if i >= 1:
            first_pdb_id = similars[0].pdb_id

            # Align
            pm.align(sim.pdb_id, first_pdb_id)

            # Check nearby non-prosthetic ligands
            # Apo detection
            if ligand:
                model = pm.get_model(
                    f"({sim.pdb_id} and ({compounds}))"
                    f" within {dist} of"
                    f"({first_pdb_id} and (resn {ligand}))"
                )
                resns = set(a.resn for a in model.atom)
                is_apo = True
                for resn in resns:
                    if resn not in prosthetic_groups.split():
                        is_apo = False
                        break
                if not is_apo:
                    pm.delete(sim.pdb_id)
                    continue

        # Set the score property
        if hasattr(pm, "set_property"):
            pm.set_property("shape3d_score", sim.score, sim.pdb_id)

        # Go on
        similars.append(sim)
    return similars



import webbrowser
from ftplib import FTP
from functools import lru_cache

import requests
from pymol import cmd as pm

from ..commons import rscript
from ..prefs import PLUGIN_DATA_DIR


@pm.extend
def fetch_similar_blast_update():
    """
    The cluster database needs to be updated before the first use of the
    fetch_similar_blast feature. The database ftp://resources.rcsb.org/sequence/clusters/
    is updated weekly, for new data is prudent to run this command weekly.
    """
    rscb_server = FTP("resources.rcsb.org")
    rscb_server.login()

    rscb_server.cwd("/sequence/clusters/")
    for cluster_fname in [
        "bc-100.out",
        "bc-95.out",
        "bc-90.out",
        "bc-70.out",
        "bc-50.out",
        "bc-40.out",
        "bc-30.out",
    ]:
        with open(PLUGIN_DATA_DIR + "/" + cluster_fname, "wb") as cluster_file:
            rscb_server.retrbinary(
                "RETR " + cluster_fname, cluster_file.write, blocksize=262144
            )


def find_similar_chain_ids(chain_id, threshold):
    """Fetch structure similar chain ids from RCSB PDB.
    chain_id - reference chain id
    threshold - similarity threshold
    """

    cluster_fname = f"bc-{threshold}.out"
    with open(PLUGIN_DATA_DIR + "/" + cluster_fname) as cluster_file:
        for cluster in cluster_file:
            if chain_id.upper() in cluster.upper():
                break
        else:
            return []

        sim_chain_ids = []
        for chain_id in cluster.split():
            pdb, chain = chain_id.split("_")
            sim_chain_ids.append((pdb.upper(), chain.upper()))
        return sim_chain_ids


@lru_cache()
def get_resolution(pdb_id):
    """
    Get the resolution for a PDB id, or None case it doesn't have.
    """
    ret = requests.post(
        "https://data.rcsb.org/graphql",
        json={
            "query": f"""
    {{
        entry(entry_id: "{pdb_id}") {{
            pdbx_vrpt_summary {{
                PDB_resolution
            }}
      }}
    }}
    """
        },
    )
    data = ret.json()
    resol = data["data"]["entry"]["pdbx_vrpt_summary"]["PDB_resolution"]
    return resol


def plot_hierarquical_cluster(chain_ids):
    chain_ids = ",".join(map(repr, chain_ids))
    out, success = rscript(
        f"""
        library(bio3d)
        ids <- c({chain_ids})
        files <- get.pdb(ids, path="pdbs", split=TRUE)
        pdbs <- pdbaln(files)
        cores <- core.find(pdbs)
        xyz <- pdbfit(pdbs, inds=cores)
        pc <- pca(xyz, rm.gaps=TRUE)
        d <- dist(pc$z[, 1:2])
        hc <- hclust(d)
        hclustplot(hc, k=1, labels = ids)
    """
    )
    webbrowser.open("Rplots.pdf")
    print(out)


@pm.extend
def fetch_similar_blast(
    chain_id,
    similarity=95,
    ligand=None,
    dist=5,
    compounds="organic or inorganic",
    prosthetic_groups="HEM FAD NAP NDP ADP FMN",
    max_resolution=None,
    max_structures=50,
):
    """
    Fetch sequence similar structures from RCSB PDB and optionally keep only
    apo structures. Apo are evaluated respective to a choosen ligand on the
    reference chain.

    On the first use update the database with the command `update_cluster_data`.
    Update the database weekly.

    OPTIONS
        chain_id        Reference structure chain id.
        similarity      Sequence similarity threshold (one of the available
                        from RCSB PDB).
        ligand          Reference ligand PDB id.
        dist            Distance cut-off around reference ligand for apo
                        evaluation.
        compounds       Selection that should be considered ligands upon apo
                        computation. Only used when ligand is given.
        prothestic_groups   List of ligands to be ignored when evaluating apo.
        max_resolution  Fetch only X-ray structures with up to such
                        resolution.
        max_structures  Fetch at most n structures. 0 for all structures.
    EXAMPLES
        fetch_similar_blast 2XY9_A, 100
        fetch_similar_blast 2XY9_A, 95, 3ES, 3, organic
        fetch_similar_blast 6Y2F_A, max_structures=0
    SEE ALSO
        update_cluster_data
        fetch_similar_shape3d
    """
    chain_id = chain_id.upper()
    max_structures = int(max_structures)
    pm.fetch(chain_id, chain_id)

    sims = []
    similars = find_similar_chain_ids(chain_id, similarity)
    cont = 0
    for sim_pdb, sim_chain in similars:

        if max_structures != 0 and cont >= max_structures:
            break

        sim_obj = f"{sim_pdb}_{sim_chain}"
        if sim_obj.upper() == chain_id.upper():
            continue

        pm.fetch(sim_obj, **{"async": 0})
        pm.align(sim_obj, chain_id)

        # Check the resolution
        resol = None
        if max_resolution:
            resol = get_resolution(sim_pdb)
            if not resol or resol > max_resolution:
                pm.delete(sim_obj)
                continue

        # Check nearby ligands
        if ligand:
            model = pm.get_model(
                f"({sim_obj} and ({compounds}))"
                f" within {dist} of"
                f"({chain_id} and (resn {ligand}))"
            )
            resns = set(a.resn for a in model.atom)

            is_apo = True
            for resn in resns:
                if resn not in prosthetic_groups.split():
                    is_apo = False
                    break

            if not is_apo:
                pm.delete(sim_obj)
                continue

        cont += 1
        sims.append((sim_obj, sim_chain, sim_pdb, resol))

    plot_hierarquical_cluster([chain_id] + [s[0] for s in sims])
    return sims



#!/usr/bin/env python

import pathlib

from setuptools import find_packages, setup

HERE = pathlib.Path(__file__).parent
README = (HERE / "README.md").read_text()


setup(
    name="pymol-labimm",
    version="0.13.6",
    description="Some PyMOL utilities",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/pslacerda/pymol-labimm",
    author="Pedro Sousa Lacerda",
    author_email="pslacerda@gmail.com",
    license="MIT",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "License :: OSI Approved :: MIT License",
        "Environment :: Plugins",
        "Intended Audience :: Science/Research",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    packages=find_packages(),
    install_requires=[
        "lxml",
        "pandas",
        "scipy",
        "requests",
        "cached_property",
        "matplotlib",
        "seaborn",
        "jinja2",
    ],
)



import os.path
import platform

import pymol.plugins

if platform.platform() == "MacOS":
    PREF_GUESSES = {
        "LABIMM_VINA": "",
        "LABIMM_OBABEL": "",
        "LABIMM_ADT_PYTHON": "",
        "LABIMM_PREPARE_RECEPTOR": "",
        "LABIMM_PREPARE_FLEXRECEPTOR": "",
        "LABIMM_RSCRIPT": "",
    }

elif platform.platform() == "Windows":
    PREF_GUESSES = {
        "LABIMM_VINA": "",
        "LABIMM_OBABEL": "",
        "LABIMM_ADT_PYTHON": "",
        "LABIMM_PREPARE_RECEPTOR": "",
        "LABIMM_PREPARE_FLEXRECEPTOR": "",
        "LABIMM_RSCRIPT": "",
    }

else:  # platform == 'Linux'
    PREFS_GUESSES = {
        "LABIMM_VINA": "/usr/bin/vina",
        "LABIMM_OBABEL": "/usr/bin/obabel",
        "LABIMM_ADT_PYTHON": "/usr/bin/python2.7",
        "LABIMM_PREPARE_RECEPTOR": "/usr/lib/python2.7/dist-packages/AutoDockTools/Utilities24/prepare_receptor4.py",
        "LABIMM_PREPARE_FLEXRECEPTOR": "/usr/lib/python2.7/dist-packages/AutoDockTools/Utilities24/prepare_flexreceptor4.py",
        "LABIMM_RSCRIPT": "/usr/bin/Rscript",
    }


def guess_prefs():
    for pref in PREFS_GUESSES:
        if not pymol.plugins.pref_get(pref):
            if os.path.exists(PREFS_GUESSES[pref]):
                pymol.plugins.pref_set(pref, PREFS_GUESSES[pref])


PLUGIN_DATA_DIR = os.path.expanduser("~/.pymol/labimm")
os.makedirs(PLUGIN_DATA_DIR, exist_ok=True)
